-----------------------------------------------------------------------------------------------
-- Client Lua Script for CombatProficiencyTheory
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- CombatProficiencyTheory Module Definition
-----------------------------------------------------------------------------------------------
local CombatProficiencyTheory = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local knBaseHit = 0.9
local knBonusPowerPerLevel = 20
local knRaidBossCritChance = 0.13
local knBaseDamage = 1
local knBossCritDamageBonus = 1.5
local knAssaultNormalizingModifier = 1.37
local knTankingNormalizingModifier = 2.8
local knHealingNormalizingModifer = 3.75
local iconTankProf = "IconSprites:Icon_Windows_UI_CRB_Attribute_CriticalHit"
local iconAttackProf = "IconSprites:Icon_Windows_UI_CRB_Attribute_PvPOffense"
local iconHealingProf = "IconSprites:Icon_Windows_UI_CRB_Attribute_Insight"
local iconDefault = "CRB_Anim_Spinner:sprAnim_SpinnerLarge"
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CombatProficiencyTheory:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function CombatProficiencyTheory:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function CombatProficiencyTheory:OnSave(eLevel)
	-- We save at a character level in case of alts in different guilds
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return
	end

	tSave = {}
	--local tSavedExportValues = {nTimeLastExported,strGuild,tHeaders,tRosterData}
	if tCPTMeta ~= nil then
		tSave = tCPTMeta
	end
	--local tSave = tSavedExportValues
	
	return tSave
end

function CombatProficiencyTheory:OnRestore(eLevel, tSaveData)
if tSaveData.ExportTime ~= nil then 
		self.tCPTMeta = tSaveData
end
end
 

-----------------------------------------------------------------------------------------------
-- CombatProficiencyTheory OnLoad
-----------------------------------------------------------------------------------------------
function CombatProficiencyTheory:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CombatProficiencyTheory.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- CombatProficiencyTheory OnDocLoaded
-----------------------------------------------------------------------------------------------
function CombatProficiencyTheory:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CombatProficiencyTheoryForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("cpt", 							"OnCPTOn", self)
		-- Deprecated: Apollo.RegisterSlashCommand("gseval",							"OnCPTEval",self)
		-- Deprecated: Apollo.RegisterEventHandler("MouseOverUnitChanged",				"OnMouseOverUnitChanged",self)
		Apollo.RegisterEventHandler("ToggleCharacterWindow", 			"OnToggleCharacterWindow", self)
		Apollo.RegisterEventHandler("WindowManagementReady", 			"OnWindowManagementReady", self)
		Apollo.RegisterEventHandler("Inspect", 							"OnInspect",self)
		
		self.timerDelay = ApolloTimer.Create(1.0, true, "OnTimer", self)
		self.chanCPT = ICCommLib.JoinChannel("ProficiencyTheoryChannel", "OnCPTMessage", self)
		-- Do additional Addon initialization here
		if not self.tCPTMeta then
			tCPTMeta = {}
		else tCPTMeta = self.tCPTMeta end
		
		--[[local CharHook = Apollo.GetAddon("Character")
        self.wndCharSheet = CharHook.wndCharacter
		self.wndEmbed = Apollo.LoadForm(self.xmlDoc,"wndCPTEmbed",self.wndCharSheet, self)
		self.wndEmbed:SetAnchorOffsets(328,178,404,243)]]--
	end
end


-----------------------------------------------------------------------------------------------
-- CombatProficiencyTheory Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/cpt"
function CombatProficiencyTheory:OnCPTOn()
	-- Disabled for now self.wndMain:Invoke() -- show the window
	
	
end

-- on timer
function CombatProficiencyTheory:OnTimer()
	-- Do your timer-related stuff here.
end

function CombatProficiencyTheory:OnTimer()
	if GameLib.GetPlayerUnit() then 
		self.timerDelay = nil
		self:OnPreloadComplete() 
	else self.timerDelay:Start()
	end
end

--combat effectiveness calc   AP/SP * hit + (AP/SP * crit * severity) * class bonus

function CombatProficiencyTheory:WSWindowCalc(unit)
	tTargetUnit = unit
	if tTargetUnit == GameLib.GetUnit().IsPlayer then
		self:OnGSEval(GameLib.GetUnit())
	end
end

function CombatProficiencyTheory:OnPreloadComplete()
	local tPlayerUnit = GameLib.GetPlayerUnit()
	if not tCPTMeta.ExportTime then
		tCPTMeta = { }
	end
	if not tCPTMeta.ExportTime or (Time.Now() - tCPTMeta.ExportTime > 3600) then 
		local nClassId = tPlayerUnit:GetClassId()
		local nExportTime = Time.Now()
		local strPlayerClass = GameLib.CodeEnumClass[nClassId]
		local nPlayerLevel = tPlayerUnit:GetLevel() 
		local nCombatEfficacy = ""
		local strName = tPlayerUnit:GetName()
		tCPTValues = self:CombatEffectiveness(tPlayerUnit)
		local strProficiencyLabel = tCPTValues.Proficiency
		local nCombatEfficacy = tCPTValues.Score 
		table.insert(tCPTMeta, {["ExportTime"] = nExportTime, ["PlayerName"] = strPlayerName, {["ClassId"] = nClassId, ["Class"] = strPlayerClass, ["Level"] = nPlayerLevel, ["Proficiency"] = strProficiencyLabel, ["Efficiacy"] = nCombatEfficacy}})
	end
	self.chanCPT:SendMessage(tCPTMeta)
end

function CombatProficiencyTheory:CombatEffectiveness(tPlayerUnit)
	local nPlayerLevel = tPlayerUnit:GetLevel()
	if nPlayerLevel < 5 then return {["Proficiency"] = "TBD", ["Score"] = "TBD", ["Icon"] = iconDefault}end
	local nAP = tPlayerUnit:GetAssaultPower()
	local nSP = tPlayerUnit:GetSupportPower()
	--if not tPlayerUnit:GetMana() then
	--else 
	local nMana = tPlayerUnit:GetMaxMana()
	--end
	if nAP and nSP then 
		if nAP > nSP then 
			local nPower = nAP + (tPlayerUnit:GetLevel() * knBonusPowerPerLevel)
			local nStrikethroughChance = math.min(knBaseHit + tPlayerUnit:GetStrikethroughChance(), 1)
			local nCritChance = tPlayerUnit:GetCritChance()
			local nCritSeverity = tPlayerUnit:GetCritSeverity()
			local strProficiency = "Damage"
			local iconAttack = iconAttackProf
			local nScoreCalc = math.floor((nPower + (nPower * nCritChance * (nCritSeverity - 1)) * nStrikethroughChance)*knAssaultNormalizingModifier)
			local nScore = tonumber(string.format("%.1f", nScoreCalc))
		return 	{["Proficiency"] = strProficiency, ["Score"] = nScore, ["Icon"] = iconAttack}
		elseif (nSP > nAP) and (nMana == 0) then			
			local nPower = nSP + (tPlayerUnit:GetLevel() * knBonusPowerPerLevel)
			local nStrikethroughChance = math.min(knBaseHit + tPlayerUnit:GetStrikethroughChance(), 1)
			local nCritChance = tPlayerUnit:GetCritChance()
			local nCritSeverity = tPlayerUnit:GetCritSeverity()
			local nMaxHp = tPlayerUnit:GetMaxHealth()
			local nMaxShield = tPlayerUnit:GetShieldCapacityMax()
			local nDeflectCrit = tPlayerUnit:GetDeflectCritChance()
			local nDeflectChance = tPlayerUnit:GetDeflectChance()
			local nPhysResist = tPlayerUnit:GetPhysicalMitigation()
			local nTechResist = tPlayerUnit:GetTechMitigation()
			local nMagicResist = tPlayerUnit:GetMagicMitigation()
			local nTotalHP = (nMaxHp + nMaxShield) / (300*nPlayerLevel) -- scales down contribution from HP+Shield
			local strProficiency = "Tanking"
			local iconTank = iconTankProf
			local nThreatGen = (nPower + (nPower * nCritChance * (nCritSeverity - 1)) * nStrikethroughChance)
			local nModifiedDamage = (knBaseDamage + (knBaseDamage * (knBossCritDamageBonus - 1) * math.max(0,knRaidBossCritChance - nDeflectCrit) ) ) * (1 - nDeflectChance) * (1 - ((nPhysResist + nTechResist + nMagicResist)))
			local nMitigation = 1 - (nModifiedDamage / knBaseDamage)
			local nScoreCalc = math.floor((nThreatGen * nTotalHP * nMitigation) / knTankingNormalizingModifier)
			local nScore = tonumber(string.format("%.1f",nScoreCalc))
			return {["Proficiency"] = strProficiency, ["Score"] = nScore, ["Icon"] = iconTank}
		else 
			local nPower = nSP + (tPlayerUnit:GetLevel() * knBonusPowerPerLevel)
			local nCritChance = tPlayerUnit:GetCritChance()
			local nCritSeverity = tPlayerUnit:GetCritSeverity()
			local nManaRegen = tPlayerUnit:GetManaRegenInCombat()*2*180 --180 seconds = 3 minute fight
			local nManaCostReduction = tPlayerUnit:GetManaCostModifier()
			local nMana = tPlayerUnit:GetMaxMana()
			local strProficiency = "Healing"
			local iconHealing = iconHealingProf
			local nScoreCalc = math.floor((nPower + (nPower * nCritChance * (nCritSeverity - 1)) * ( (nMana + nManaRegen)/(1 - nManaCostReduction) ) / 1000) / knHealingNormalizingModifer) --need data for this, needs to account for regen in combat
			local nScore = tonumber(string.format("%.1f", nScoreCalc))
			return {["Proficiency"] = strProficiency, ["Score"] = nScore, ["Icon"] = iconHealing}

		end
	end
end

function CombatProficiencyTheory:OnCPTEval()
	local tPlayerUnit = GameLib.GetPlayerUnit()
	self:OnMouseOverUnitChanged(tPlayerUnit)
end

function CombatProficiencyTheory:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "Combat Proficency Theory Options"})
end

-----------------------------------------------------------------------------------------------
-- WSGearScoreForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel button is clicked
function CombatProficiencyTheory:OnCancel()
	self.wndMain:Close() -- hide the window
end

function CombatProficiencyTheory:OnToggleCharacterWindow()
	local CharHook = Apollo.GetAddon("Character")
    self.wndCharSheet = CharHook.wndCharacter
	self.wndEmbed = Apollo.LoadForm(self.xmlDoc,"wndCPTEmbed",self.wndCharSheet, self)
	self.wndEmbed:SetAnchorOffsets(328,178,404,243)
	local tPlayerUnit = GameLib.GetPlayerUnit()
	tCPTValues = self:CombatEffectiveness(tPlayerUnit)
	local strPlayerProficiency = tCPTValues.Proficiency
	local nPlayerScore = tCPTValues.Score 
	local iconProf = tCPTValues.Icon
	self.wndEmbed:FindChild("Proficiency"):SetText(strPlayerProficiency)
	self.wndEmbed:FindChild("CPTData"):SetText(nPlayerScore)
	self.wndEmbed:FindChild("ProfIcon"):SetSprite(iconProf)
	if tPlayerUnit:GetLevel() < 5 then 
		self.wndEmbed:SetTooltip("Your character has not progressed enough to estimate proficiency in any spec.")
	else
		self.wndEmbed:SetTooltip("Based on your primary & secondary attributes, this value represents an estimate of your effectiveness when spec'd for "..strPlayerProficiency..". \n \nThis estimate is derived from your base stats and contributions from items and other stat enhancers.")
	end
end

function CombatProficiencyTheory:OnInspect(unitInspecting, arItems)
	local InspectHook = Apollo.GetAddon("Inspect")
    self.wndInspectSheet= InspectHook.wndCharacter
	self.wndInspectEmbed = Apollo.LoadForm(self.xmlDoc,"wndCPTInspectEmbed",self.wndInspectSheet, self)
	self.wndInspectEmbed:SetAnchorOffsets(329,144,405,209)
	local tPlayerUnit = unitInspecting
	tCPTInspectValues = self:CombatEffectiveness(tPlayerUnit)
	local strPlayerProficiency = tCPTInspectValues.Proficiency
	local nPlayerScore = tCPTInspectValues.Score 
	local iconProf = tCPTInspectValues.Icon
	self.wndInspectEmbed:FindChild("Proficiency"):SetText(strPlayerProficiency)
	self.wndInspectEmbed:FindChild("CPTData"):SetText(nPlayerScore)
	self.wndInspectEmbed:FindChild("ProfIcon"):SetSprite(iconProf)
	if tPlayerUnit:GetLevel() < 5 then 
		self.wndInspectEmbed:SetTooltip("This character has not progressed enough to estimate their proficiency in any spec.")
	else
		self.wndInspectEmbed:SetTooltip("Based on primary & secondary attributes, this player's estimated proficiency is: "..nPlayerScore.." when spec'd for "..strPlayerProficiency..". \n \nThis value is derived from their base stats and contributions from items and other stat enhancers.")
	end	
end

-----------------------------------------------------------------------------------------------
-- CombatProficiencyTheory Instance
-----------------------------------------------------------------------------------------------
local CombatProficiencyTheoryInst = CombatProficiencyTheory:new()
CombatProficiencyTheory:Init()